@@ -14,18 +14,17 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2014, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <errno.h>
 
-#include <gdk/gdkkeysyms.h>
 #include <pango/pango.h>
 
 #include "yad.h"
 
 static GtkWidget *text_view;
-static GtkTextBuffer *text_buffer;
+static GObject *text_buffer;
 static GtkTextTag *tag;
 static GdkCursor *hand, *normal;
 static gchar *pattern = NULL;
@@ -46,12 +45,12 @@ do_search (GtkWidget * e, GtkWidget * w)
   gtk_widget_destroy (w);
   gtk_widget_queue_draw (text_view);
 
-  if (new_search || gtk_text_buffer_get_modified (text_buffer))
+  if (new_search || gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (text_buffer)))
     {
       /* get the text */
       g_free (text);
-      gtk_text_buffer_get_bounds (text_buffer, &begin, &end);
-      text = gtk_text_buffer_get_text (text_buffer, &begin, &end, FALSE);
+      gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &begin, &end);
+      text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &begin, &end, FALSE);
       offset = 0;
       /* compile new regex */
       if (regex)
@@ -71,10 +70,10 @@ do_search (GtkWidget * e, GtkWidget * w)
       spos = g_utf8_pointer_to_offset (text, text + sp + offset);
       epos = g_utf8_pointer_to_offset (text, text + ep + offset);
 
-      gtk_text_buffer_get_iter_at_offset (text_buffer, &begin, spos);
-      gtk_text_buffer_get_iter_at_offset (text_buffer, &end, epos);
+      gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &begin, spos);
+      gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &end, epos);
 
-      gtk_text_buffer_select_range (text_buffer, &begin, &end);
+      gtk_text_buffer_select_range (GTK_TEXT_BUFFER (text_buffer), &begin, &end);
       gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &begin, 0, FALSE, 0, 0);
 
       offset += epos;
@@ -170,7 +169,7 @@ key_press_cb (GtkWidget * w, GdkEventKey * key, gpointer data)
 #endif
     {
       if (options.plug == -1)
-        gtk_dialog_response (GTK_DIALOG (data), YAD_RESPONSE_OK);
+        yad_exit (YAD_RESPONSE_OK);
       return TRUE;
     }
 
@@ -204,7 +203,7 @@ tag_event_cb (GtkTextTag * tag, GObject * obj, GdkEvent * ev, GtkTextIter * iter
           gtk_text_iter_forward_to_tag_toggle (&end, tag);
 
           url = gtk_text_iter_get_text (&start, &end);
-          cmdline = g_strdup_printf ("xdg-open '%s'", url);
+          cmdline = g_strdup_printf (settings.open_cmd, url);
           g_free (url);
 
           g_spawn_command_line_async (cmdline, NULL);
@@ -332,35 +331,35 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
         }
 
       if (string->str[0] == '\014')
-	{
+        {
           GtkTextIter start, end;
 
-	  /* clear text if ^L received */
-          gtk_text_buffer_get_start_iter (text_buffer, &start);
-          gtk_text_buffer_get_end_iter (text_buffer, &end);
-          gtk_text_buffer_delete (text_buffer, &start, &end);
-	}
+          /* clear text if ^L received */
+          gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (text_buffer), &start);
+          gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (text_buffer), &end);
+          gtk_text_buffer_delete (GTK_TEXT_BUFFER (text_buffer), &start, &end);
+        }
       else if (string->len > 0)
         {
           GtkTextIter end;
 
-          gtk_text_buffer_get_end_iter (text_buffer, &end);
+          gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (text_buffer), &end);
 
           if (!g_utf8_validate (string->str, string->len, NULL))
             {
               gchar *utftext =
                 g_convert_with_fallback (string->str, string->len, "UTF-8", "ISO-8859-1", NULL, NULL, NULL, NULL);
-              gtk_text_buffer_insert (text_buffer, &end, utftext, -1);
+              gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, utftext, -1);
               g_free (utftext);
             }
           else
-            gtk_text_buffer_insert (text_buffer, &end, string->str, string->len);
+            gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, string->str, string->len);
 
           if (options.text_data.tail)
             {
               while (gtk_events_pending ())
                 gtk_main_iteration ();
-              gtk_text_buffer_get_end_iter (text_buffer, &end);
+              gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (text_buffer), &end);
               gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &end, 0, FALSE, 0, 0);
             }
         }
@@ -368,6 +367,15 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
       g_string_free (string, TRUE);
     }
 
+#ifdef HAVE_SOURCEVIEW
+  if (options.source_data.lang)
+    {
+      GtkSourceLanguage *lang = gtk_source_language_manager_get_language (gtk_source_language_manager_get_default (),
+                                                                          options.source_data.lang);
+      gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (text_buffer), lang);
+    }
+#endif
+
   return TRUE;
 }
 
@@ -375,6 +383,9 @@ static void
 fill_buffer_from_file ()
 {
   GtkTextIter iter, end;
+#ifdef HAVE_SOURCEVIEW
+  GtkSourceLanguage *lang;
+#endif
   FILE *f;
   gchar buf[2048];
   gint remaining = 0;
@@ -390,7 +401,7 @@ fill_buffer_from_file ()
       return;
     }
 
-  gtk_text_buffer_get_iter_at_offset (text_buffer, &iter, 0);
+  gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &iter, 0);
 
   while (!feof (f))
     {
@@ -404,7 +415,7 @@ fill_buffer_from_file ()
       g_utf8_validate (buf, count + remaining, &leftover);
 
       g_assert (g_utf8_validate (buf, leftover - buf, NULL));
-      gtk_text_buffer_insert (text_buffer, &iter, buf, leftover - buf);
+      gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &iter, buf, leftover - buf);
 
       remaining = (buf + remaining + count) - leftover;
       memmove (buf, leftover, remaining);
@@ -423,17 +434,17 @@ fill_buffer_from_file ()
    * a newline, so we delete to the end of the buffer to clean up.
    */
 
-  gtk_text_buffer_get_end_iter (text_buffer, &end);
-  gtk_text_buffer_delete (text_buffer, &iter, &end);
-  gtk_text_buffer_set_modified (text_buffer, FALSE);
+  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (text_buffer), &end);
+  gtk_text_buffer_delete (GTK_TEXT_BUFFER (text_buffer), &iter, &end);
+  gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (text_buffer), FALSE);
 
-  if (options.text_data.tail)
-    {
-      while (gtk_events_pending ())
-        gtk_main_iteration ();
-      gtk_text_buffer_get_end_iter (text_buffer, &end);
-      gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (text_view), &end, 0, FALSE, 0, 0);
-    }
+#ifdef HAVE_SOURCEVIEW
+  if (options.source_data.lang)
+    lang = gtk_source_language_manager_get_language (gtk_source_language_manager_get_default (), options.source_data.lang);
+  else
+    lang = gtk_source_language_manager_guess_language (gtk_source_language_manager_get_default (), options.common_data.uri, NULL);
+  gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (text_buffer), lang);
+#endif
 }
 
 static void
@@ -451,20 +462,25 @@ GtkWidget *
 text_create_widget (GtkWidget * dlg)
 {
   GtkWidget *w;
+  PangoFontDescription *fd;
 
   w = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w), GTK_SHADOW_ETCHED_IN);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), options.hscroll_policy, options.vscroll_policy);
 
-  text_view = gtk_text_view_new ();
+#ifdef HAVE_SOURCEVIEW
+  text_buffer = (GObject *) gtk_source_buffer_new (NULL);
+  text_view = gtk_source_view_new_with_buffer (GTK_SOURCE_BUFFER (text_buffer));
+#else
+  text_buffer = (GObject *) gtk_text_buffer_new (NULL);
+  text_view = gtk_text_view_new_with_buffer (GTK_TEXT_BUFFER (text_buffer));
+#endif
   gtk_widget_set_name (text_view, "yad-text-widget");
-  text_buffer = gtk_text_buffer_new (NULL);
-  gtk_text_view_set_buffer (GTK_TEXT_VIEW (text_view), text_buffer);
   gtk_text_view_set_justification (GTK_TEXT_VIEW (text_view), options.text_data.justify);
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (text_view), options.text_data.margins);
   gtk_text_view_set_right_margin (GTK_TEXT_VIEW (text_view), options.text_data.margins);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), options.common_data.editable);
-  if (!options.common_data.editable)
+  if (!options.common_data.editable && options.text_data.hide_cursor)
     gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), FALSE);
 
   if (options.text_data.wrap)
@@ -487,7 +503,7 @@ text_create_widget (GtkWidget * dlg)
     {
 #if GTK_CHECK_VERSION(3,0,0)
       GdkRGBA clr;
-      if (gdk_rgba_parse (&clr, options.text_data.fore))
+      if (gdk_rgba_parse (&clr, options.text_data.back))
         gtk_widget_override_background_color (text_view, GTK_STATE_FLAG_NORMAL, &clr);
 #else
       GdkColor clr;
@@ -496,16 +512,27 @@ text_create_widget (GtkWidget * dlg)
 #endif
     }
 
+  /* set font */
   if (options.common_data.font)
-    {
-      PangoFontDescription *fd = pango_font_description_from_string (options.common_data.font);
+    fd = pango_font_description_from_string (options.common_data.font);
+  else
+    fd = pango_font_description_from_string ("Monospace");
+
 #if GTK_CHECK_VERSION(3,0,0)
-      gtk_widget_override_font (text_view, fd);
+  gtk_widget_override_font (text_view, fd);
 #else
-      gtk_widget_modify_font (text_view, fd);
+  gtk_widget_modify_font (text_view, fd);
 #endif
-      pango_font_description_free (fd);
+  pango_font_description_free (fd);
+
+#ifdef HAVE_SPELL
+  if (options.common_data.enable_spell)
+    {
+      GtkSpellChecker *spell = gtk_spell_checker_new ();
+      gtk_spell_checker_set_language (spell, options.common_data.spell_lang, NULL);
+      gtk_spell_checker_attach (spell, GTK_TEXT_VIEW (text_view));
     }
+#endif
 
   /* Add submit on ctrl+enter */
   g_signal_connect (text_view, "key-press-event", G_CALLBACK (key_press_cb), dlg);
@@ -519,8 +546,10 @@ text_create_widget (GtkWidget * dlg)
                            G_REGEX_CASELESS | G_REGEX_OPTIMIZE | G_REGEX_EXTENDED, G_REGEX_MATCH_NOTEMPTY, NULL);
 
       /* Create text tag for URI */
-      tag = gtk_text_buffer_create_tag (text_buffer, NULL,
-                                        "foreground", "blue", "underline", PANGO_UNDERLINE_SINGLE, NULL);
+      tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (text_buffer), NULL,
+                                        "foreground", options.text_data.uri_color,
+                                        "underline", PANGO_UNDERLINE_SINGLE,
+                                        NULL);
       g_object_set_data (G_OBJECT (tag), "is_link", GINT_TO_POINTER (1));
       g_signal_connect (G_OBJECT (tag), "event", G_CALLBACK (tag_event_cb), NULL);
 
@@ -539,6 +568,14 @@ text_create_widget (GtkWidget * dlg)
 
   if (options.common_data.listen || options.common_data.uri == NULL)
     fill_buffer_from_stdin ();
+  else
+    {
+      /* place cursor at start of file */
+      GtkTextIter iter;
+
+      gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (text_buffer), &iter, 0);
+      gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (text_buffer), &iter);
+    }
 
   return w;
 }
@@ -552,8 +589,8 @@ text_print_result (void)
   if (!options.common_data.editable)
     return;
 
-  gtk_text_buffer_get_bounds (text_buffer, &start, &end);
-  text = gtk_text_buffer_get_text (text_buffer, &start, &end, 0);
+  gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (text_buffer), &start, &end);
+  text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (text_buffer), &start, &end, 0);
   g_print ("%s", text);
   g_free (text);
 }
